//--------------------------------------------------------------------
// UTCN, Departamentul de Calculatoare
// Sisteme de intrare/iesire si echipamente periferice
//--------------------------------------------------------------------
// http://users.utcluj.ro/~baruch/ro/pages/cursuri/sisteme-de-intrareiesire/laborator.php
//--------------------------------------------------------------------
// Fisier:		PCI.h
// Data:		09.08.2015
// Modificat:	06.03.2021
//--------------------------------------------------------------------
// Definitii pentru accesul la spatiul de configuratie PCI/PCIe 
//
// Sursele informatiilor:
// - PCI Code and ID Assignment Specification, Revision 1.11, 
//   PCI-SIG, Jan. 24, 2019.
// - The PCI ID Repository, http://pci-ids.ucw.cz/
//--------------------------------------------------------------------

#ifndef _PCI_H
#define _PCI_H

#define PCI_CONFIG_ADR		0x0CF8		// Port de adresa pentru accesul in spatiul de configuratie
#define PCI_CONFIG_DATA		0x0CFC		// Port de date pentru accesul in spatiul de configuratie

// Structuri pentru registrele de configuratie PCI/PCIe

// Structura pentru antetul de tip 0
typedef struct tPCI_CONFIG0 {
	WORD		VendorID;				// 0x00 Vendor ID
	WORD		DeviceID;				// 0x02 Device ID
	WORD		Command;				// 0x04 Command Register
	WORD		Status;					// 0x06 Status Register
	BYTE		RevisionID;				// 0x08 Revision ID
	BYTE		ProgInterface;			// 0x09 Programming Interface
	BYTE		SubClass;				// 0x0A Sub-Class Code
	BYTE		BaseClass;				// 0x0B Base Class-Code
	BYTE		CacheLineSize;			// 0x0C Cache Line Size
	BYTE		LatencyTimer;			// 0x0D Master Latency Timer (0x00 for PCIe)
	BYTE		HeaderType;				// 0x0E Header Type
	BYTE		BIST;					// 0x0F Built-In Self-Test
	DWORD		BaseAddress0;			// 0x10 Base Address 0
	DWORD		BaseAddress1;			// 0x14 Base Address 1
	DWORD		BaseAddress2;			// 0x18 Base Address 2
	DWORD		BaseAddress3;			// 0x1C Base Address 3
	DWORD		BaseAddress4;			// 0x20 Base Address 4
	DWORD		BaseAddress5;			// 0x24 Base Address 5
	DWORD		CardBusCIS;				// 0x28 CardBus CIS Pointer
	WORD		SubSystVendorID;		// 0x2C Subsystem Vendor ID
	WORD		SubSystID;				// 0x2E Subsystem ID
	DWORD		ROMBaseAddress;			// 0x30 Expansion ROM Base Address
	BYTE		CapabilitiesPtr;		// 0x34 Capabilities Pointer
	BYTE		Res0[3];				// 0x35-0x37 Reserved
	DWORD		Res1;					// 0x38 Reserved
	BYTE		InterruptLine;			// 0x3C Interrupt Line
	BYTE		InterruptPin;			// 0x3D Interrupt Pin
	BYTE		MinGrant;				// 0x3E Minimum Grant (0x00 for PCIe)
	BYTE		MaxLatency;				// 0x3F Maximum Latency (0x00 for PCIe)
	DWORD		DevSpecific[48];		// 0x40-0xFF Device-Specific Registers
} PCI_CONFIG0, *PPCI_CONFIG0;

// Structura pentru antetul de tip 1
typedef struct tPCI_CONFIG1 {
	WORD		VendorID;				// 0x00 Vendor ID
	WORD		DeviceID;				// 0x02 Device ID
	WORD		Command;				// 0x04 Command Register
	WORD		Status;					// 0x06 Status Register
	BYTE		RevisionID;				// 0x08 Revision ID
	BYTE		ProgInterface;			// 0x09 Programming Interface
	BYTE		SubClass;				// 0x0A Sub-Class Code
	BYTE		BaseClass;				// 0x0B Base Class-Code
	BYTE		CacheLineSize;			// 0x0C Cache Line Size
	BYTE		LatencyTimer;			// 0x0D Master Latency Timer (0x00 for PCIe)
	BYTE		HeaderType;				// 0x0E Header Type
	BYTE		BIST;					// 0x0F Built-In Self-Test
	DWORD		BaseAddress0;			// 0x10 Base Address 0
	DWORD		BaseAddress1;			// 0x14 Base Address 1
	BYTE		PrimaryBus;				// 0x18 Primary Bus Number
	BYTE		SecondaryBus;			// 0x19 Secondary Bus Number
	BYTE		SubordinateBus;			// 0x1A Subordinate Bus Number
	BYTE		SecondaryLatency;		// 0x1B Secondary Latency Timer
	BYTE		IOBase;					// 0x1C I/O Base
	BYTE		IOLimit;				// 0x1D I/O Limit
	WORD		SecondaryStatus;		// 0x1E Secondary Status Register
	WORD		MemoryBase;				// 0x20 Memory Base
	WORD		MemoryLimit;			// 0x22 Memory Limit
	WORD		PrefMemoryBase;			// 0x24 Prefetchable Memory Base
	WORD		PrefMemoryLimit;		// 0x26 Prefetchable Memory Limit
	DWORD		PrefBaseUpper32;		// 0x28 Prefetchable Base Upper 32 Bits
	DWORD		PrefLimitUpper32;		// 0x2C Prefetchable Limit Upper 32 Bits
	WORD		IOBaseUpper16;			// 0x30 I/O Base Upper 16 Bits
	WORD		IOLimitUpper16;			// 0x32 I/O Limit Upper 16 Bits
	BYTE		CapabilitiesPtr;		// 0x34 Capabilities Pointer
	BYTE		Res0[3];				// 0x35-0x37 Reserved
	DWORD		ROMBaseAddress;			// 0x38 Expansion ROM Base Address
	BYTE		InterruptLine;			// 0x3C Interrupt Line
	BYTE		InterruptPin;			// 0x3D Interrupt Pin
	WORD		BridgeControl;			// 0x3E Bridge Control
	DWORD		DevSpecific[48];		// 0x40-0xFF Device-Specific Registers
} PCI_CONFIG1, *PPCI_CONFIG1;

// Structura pentru tabela codurilor de clase PCI/PCIe
typedef struct tPCI_CLASS_TABLE
{
	BYTE		Class;					// codul clasei
	BYTE		SubClass;				// codul subclasei
	BYTE		ProgIf;					// interfata de programare
	CONST CHAR* ClassDesc;				// descriptorul clasei si al subclasei
	CONST CHAR* ProgIfDesc;				// descriptorul interfetei de programare
} PCI_CLASS_TABLE, * PPCI_CLASS_TABLE;

PCI_CLASS_TABLE PciClassTable [] =
{
	0x00, 0x00, 0x00, "Dispozitiv neclasificat non-VGA", "",
	0x00, 0x01, 0x00, "Dispozitiv neclasificat compatibil VGA", "",
	0x00, 0x05, 0x00, "Coprocesor de imagini", "",

	0x01, 0x00, 0x00, "Controler SCSI ", "(interfata specifica producatorului)",
	0x01, 0x00, 0x11, "Dispozitiv de stocare SCSI - SCSI prin port PCI Express ", "(PCI Express Queuing Interface)",
	0x01, 0x00, 0x12, "Controler SCSI - SCSI prin port PCI Express ", "(PCI Express Queuing Interface)",
	0x01, 0x00, 0x13, "Dispozitiv de stocare SCSI si controler SCSI - SCSI prin port PCI Express ", "(PCI Express Queuing Interface)",
	0x01, 0x00, 0x21, "Dispozitiv de stocare SCSI - SCSI prin port PCI Express ", "(interfata NVM Express)",

	0x01, 0x01, 0x00, "Controler IDE ", "(numai mod compatibil ISA)",
	0x01, 0x01, 0x05, "Controler IDE ", "(numai mod nativ PCI)",
	0x01, 0x01, 0x0A, "Controler IDE ", "(mod compatibil ISA)",
	0x01, 0x01, 0x0F, "Controler IDE ", "(mod nativ PCI)",
	0x01, 0x01, 0x80, "Controler IDE ", "(numai mod compatibil ISA cu functie master de magistrala)",
	0x01, 0x01, 0x85, "Controler IDE ", "(numai mod nativ PCI cu functie master de magistrala)",
	0x01, 0x01, 0x8A, "Controler IDE ", "(mod compatibil ISA cu functie master de magistrala)",
	0x01, 0x01, 0x8F, "Controler IDE ", "(mod nativ PCI cu functie master de magistrala)",
	0x01, 0x02, 0x00, "Controler de disc flexibil ", "(interfata specifica producatorului)",
	0x01, 0x03, 0x00, "Controler de magistrala IPI (Intelligent Peripheral Interface) ", "(interfata specifica producatorului)",
	0x01, 0x04, 0x00, "Controler RAID ", "(interfata specifica producatorului)",
	0x01, 0x05, 0x20, "Controler ATA ", "(interfata ADMA, transfer singular)",
	0x01, 0x05, 0x30, "Controler ATA ", "(interfata ADMA, functionare continua)",
	0x01, 0x06, 0x00, "Controler Serial ATA (SATA) ", "(interfata specifica producatorului)",
	0x01, 0x06, 0x01, "Controler Serial ATA (SATA) ", "(interfata AHCI)",
	0x01, 0x06, 0x02, "Controler Serial ATA (SATA) ", "(interfata Serial Storage Bus)",
	0x01, 0x07, 0x00, "Controler Serial Attached SCSI (SAS) ", "(interfata specifica producatorului)",
	0x01, 0x07, 0x01, "Controler Serial Attached SCSI (SAS) ", "(interfata Serial Storage Bus)",
	0x01, 0x08, 0x00, "Controler de memorie nevolatila ", "(interfata specifica producatorului)",
	0x01, 0x08, 0x01, "Controler de memorie nevolatila ", "(interfata NVHCMI)",
	0x01, 0x08, 0x02, "Controler de I/E NVM Express (NVMe)", "",
	0x01, 0x08, 0x03, "Controler de administrare NVM Express (NVMe)", "",
	0x01, 0x80, 0x00, "Alt controler de stocare masiva ", "(interfata specifica producatorului)",

	0x02, 0x00, 0x00, "Controler de retea Ethernet", "",
	0x02, 0x01, 0x00, "Controler de retea Token Ring", "",
	0x02, 0x02, 0x00, "Controler de retea FDDI", "",
	0x02, 0x03, 0x00, "Controler de retea ATM", "",
	0x02, 0x04, 0x00, "Controler de retea ISDN", "",
	0x02, 0x05, 0x00, "Controler de retea WorldFip", "",
	0x02, 0x06, 0x00, "Controler de retea PICMG 2.14 Multi Computing", "",
	0x02, 0x07, 0x00, "Controler de retea InfiniBand", "",
	0x02, 0x08, 0x00, "Controler de retea structurala ", "(interfata specifica producatorului)",
	0x02, 0x80, 0x00, "Alt controler de retea", "",

	0x03, 0x00, 0x00, "Controler de afisaj compatibil VGA", "",
	0x03, 0x00, 0x01, "Controler de afisaj compatibil 8514", "",
	0x03, 0x01, 0x00, "Controler de afisaj compatibil XGA", "",
	0x03, 0x02, 0x00, "Controler de afisaj 3D", "",
	0x03, 0x80, 0x00, "Alt controler de afisaj", "",

	0x04, 0x00, 0x00, "Dispozitiv multimedia video ", "(interfata specifica producatorului)",
	0x04, 0x01, 0x00, "Dispozitiv multimedia audio ", "(interfata specifica producatorului)",
	0x04, 0x02, 0x00, "Dispozitiv de telefonie pe calculator ", "(interfata specifica producatorului)",
	0x04, 0x03, 0x00, "Dispozitiv compatibil High Definition Audio (HD-A) 1.0", "",
	0x04, 0x03, 0x80, "Dispozitiv compatibil High Definition Audio (HD-A) 1.0 ", "(extensii suplimentare specifice producatorului)",
	0x04, 0x80, 0x00, "Alt dispozitiv multimedia ", "(interfata specifica producatorului)",

	0x05, 0x00, 0x00, "Controler de memorie RAM", "",
	0x05, 0x01, 0x00, "Controler de memorie flash", "",
	0x05, 0x80, 0x00, "Alt controler de memorie", "",

	0x06, 0x00, 0x00, "Punte calculator gazda", "",
	0x06, 0x01, 0x00, "Punte ISA", "",
	0x06, 0x02, 0x00, "Punte EISA", "",
	0x06, 0x03, 0x00, "Punte MicroChannel (MCA)", "",
	0x06, 0x04, 0x00, "Punte PCI-PCI ", "(decodificare normala)",
	0x06, 0x04, 0x01, "Punte PCI-PCI ", "(decodificare substractiva)",
	0x06, 0x05, 0x00, "Punte PCMCIA", "",
	0x06, 0x06, 0x00, "Punte NuBus", "",
	0x06, 0x07, 0x00, "Punte CardBus", "",
	0x06, 0x08, 0x00, "Punte RACEway ", "(mod transparent)",
	0x06, 0x08, 0x01, "Punte RACEway ", "(mod punct terminal)",
	0x06, 0x09, 0x40, "Punte semitransparenta PCI-PCI ", "(magistrala PCI primara catre UCP gazda)",
	0x06, 0x09, 0x80, "Punte semitransparenta PCI-PCI ", "(magistrala PCI secundara catre UCP gazda)",
	0x06, 0x0A, 0x00, "Punte calculator gazda InfiniBand-PCI", "",
	0x06, 0x0B, 0x00, "Punte calculator gazda Advanced Switching-PCI ", "(interfata specifica)",
	0x06, 0x0B, 0x01, "Punte calculator gazda Advanced Switching-PCI ", "(interfata definita de ASI-SIG)",
	0x06, 0x80, 0x00, "Alt dispozitiv punte", "",

	0x07, 0x00, 0x00, "Controler serial generic compatibil XT", "",
	0x07, 0x00, 0x01, "Controler serial compatibil 16450", "",
	0x07, 0x00, 0x02, "Controler serial compatibil 16550", "",
	0x07, 0x00, 0x03, "Controler serial compatibil 16650", "",
	0x07, 0x00, 0x04, "Controler serial compatibil 16750", "",
	0x07, 0x00, 0x05, "Controler serial compatibil 16850", "",
	0x07, 0x00, 0x06, "Controler serial compatibil 16950", "",
	0x07, 0x01, 0x00, "Port paralel ", "(SPP)",
	0x07, 0x01, 0x01, "Port paralel ", "(bidirectional)",
	0x07, 0x01, 0x02, "Port paralel ", "(compatibil ECP 1.x)",
	0x07, 0x01, 0x03, "Controler de comunicatie paralela IEEE 1284", "",
	0x07, 0x01, 0xFE, "Dispozitiv de comunicatie paralela IEEE 1284", "",
	0x07, 0x02, 0x00, "Controler serial multiport", "",
	0x07, 0x03, 0x00, "Modem generic", "",
	0x07, 0x03, 0x01, "Modem compatibil Hayes ", "(interfata compatibila 16450)",
	0x07, 0x03, 0x02, "Modem compatibil Hayes ", "(interfata compatibila 16550)",
	0x07, 0x03, 0x03, "Modem compatibil Hayes ", "(interfata compatibila 16650)",
	0x07, 0x03, 0x04, "Modem compatibil Hayes ", "(interfata compatibila 16750)",
	0x07, 0x04, 0x00, "Controler GPIB (IEEE 488.1/2)", "",
	0x07, 0x05, 0x00, "Controler Smart Card", "",
	0x07, 0x80, 0x00, "Alt controler de comunicatie", "",

	0x08, 0x00, 0x00, "Controler de intreruperi programabil (PIC) generic 8259", "",
	0x08, 0x00, 0x01, "Controler de intreruperi programabil (PIC) ISA", "",
	0x08, 0x00, 0x02, "Controler de intreruperi programabil (PIC) EISA", "",
	0x08, 0x00, 0x10, "Controler de intreruperi I/O APIC", "",
	0x08, 0x00, 0x20, "Controler de intreruperi I/O(x) APIC", "",
	0x08, 0x01, 0x00, "Controler DMA generic 8237", "",
	0x08, 0x01, 0x01, "Controler DMA ISA", "",
	0x08, 0x01, 0x02, "Controler DMA EISA", "",
	0x08, 0x02, 0x00, "Contor de timp generic 8254", "",
	0x08, 0x02, 0x01, "Contor de timp ISA", "",
	0x08, 0x02, 0x02, "Contor de timp EISA", "",
	0x08, 0x02, 0x03, "Contor de timp High Performance Event Timer (HPET)", "",
	0x08, 0x03, 0x00, "Controler generic pentru ceas de timp real", "",
	0x08, 0x03, 0x01, "Controler ISA pentru ceas de timp real", "",
	0x08, 0x04, 0x00, "Controler generic PCI Hot-Plug", "",
	0x08, 0x05, 0x00, "Controler gazda SD", "",
	0x08, 0x06, 0x00, "Controler de sistem IOMMU", "",
	0x08, 0x07, 0x00, "Controler Root Complex Event Collector", "",
	0x08, 0x80, 0x00, "Alt periferic de sistem", "",

	0x09, 0x00, 0x00, "Controler pentru tastatura", "",
	0x09, 0x01, 0x00, "Digitizor (creion)", "",
	0x09, 0x02, 0x00, "Controler pentru mouse", "",
	0x09, 0x03, 0x00, "Controler pentru scanner", "",
	0x09, 0x04, 0x00, "Controler pentru port de jocuri (generic)", "",
	0x09, 0x04, 0x10, "Controler pentru port de jocuri (extins)", "",
	0x09, 0x80, 0x00, "Controler pentru alt dispozitiv de intrare", "",

	0x0A, 0x00, 0x00, "Statie de andocare generica", "",
	0x0A, 0x80, 0x00, "Alt tip de statie de andocare", "",

	0x0B, 0x00, 0x00, "Procesor 386", "",
	0x0B, 0x01, 0x00, "Procesor 486", "",
	0x0B, 0x02, 0x00, "Procesor Pentium", "",
	0x0B, 0x10, 0x00, "Procesor Alpha", "",
	0x0B, 0x20, 0x00, "Procesor PowerPC", "",
	0x0B, 0x30, 0x00, "Procesor MIPS", "",
	0x0B, 0x40, 0x00, "Coprocesor", "",
	0x0B, 0x80, 0x00, "Alt procesor", "",

	0x0C, 0x00, 0x00, "Controler de magistrala IEEE 1394 (FireWire) ", "(generic)",
	0x0C, 0x00, 0x10, "Controler de magistrala IEEE 1394 (FireWire) ", "(specificatia OpenHCI)",
	0x0C, 0x01, 0x00, "Controler de magistrala ACCESS.bus", "",
	0x0C, 0x02, 0x00, "Controler Serial Storage Architecture (SSA)", "",
	0x0C, 0x03, 0x00, "Controler Universal Serial Bus (USB) ", "(UHCI)",
	0x0C, 0x03, 0x10, "Controler Universal Serial Bus (USB) ", "(OHCI)",
	0x0C, 0x03, 0x20, "Controler Universal Serial Bus (USB) ", "(EHCI)",
	0x0C, 0x03, 0x30, "Controler Universal Serial Bus (USB) ", "(xHCI)",
	0x0C, 0x03, 0x80, "Controler Universal Serial Bus (USB) ", "(interfata de programare nespecificata)",
	0x0C, 0x03, 0xFE, "Dispozitiv Universal Serial Bus (USB)", "",
	0x0C, 0x04, 0x00, "Controler de magistrala Fibre Channel", "",
	0x0C, 0x05, 0x00, "Controler System Management Bus (SMBus)", "",
	0x0C, 0x06, 0x00, "Controler de magistrala InfiniBand", "",
	0x0C, 0x07, 0x00, "Interfata IPMI (Intelligent Platform Management Interface) SMIC (Server Management Interface Chip)", "",
	0x0C, 0x07, 0x01, "Interfata IPMI (Intelligent Platform Management Interface) pentru controler de tastatura", "",
	0x0C, 0x07, 0x02, "Interfata IPMI (Intelligent Platform Management Interface) pentru transferuri pe blocuri", "",
	0x0C, 0x08, 0x00, "Interfata SERCOS (Serial Real-Time Communication System) (IEC 61491)", "",
	0x0C, 0x09, 0x00, "Controler CANbus", "",
	0x0C, 0x0A, 0x00, "Controler MIPI I3C Host Controller Interface", "",
	0x0C, 0x80, 0x00, "Alt controler de magistrala seriala", "",

	0x0D, 0x00, 0x00, "Controler compatibil IrDA", "",
	0x0D, 0x01, 0x00, "Controler IR pentru electronica de consum", "",
	0x0D, 0x01, 0x10, "Controler radio UWB", "",
	0x0D, 0x10, 0x00, "Controler RF", "",
	0x0D, 0x11, 0x00, "Controler Bluetooth", "",
	0x0D, 0x12, 0x00, "Controler pentru banda larga (broadband)", "",
	0x0D, 0x20, 0x00, "Controler Ethernet (802.1a - 5 GHz)", "",
	0x0D, 0x21, 0x00, "Controler Ethernet (802.1b - 2.4 GHz)", "",
	0x0D, 0x40, 0x00, "Controler/modem celular", "",
	0x0D, 0x41, 0x00, "Controler/modem celular plus Ethernet (802.11)", "",
	0x0D, 0x80, 0x00, "Alt tip de controler pentru comunicatie fara fir", "",

	0x0E, 0x00, 0x00, "Controler de I/E inteligent I2O", "",
	0x0E, 0x80, 0x00, "Controler de I/E inteligent", "",

	0x0F, 0x01, 0x00, "Controler de comunicatie TV prin satelit", "",
	0x0F, 0x02, 0x00, "Controler de comunicatie audio prin satelit", "",
	0x0F, 0x03, 0x00, "Controler de comunicatie vocala prin satelit", "",
	0x0F, 0x04, 0x00, "Controler de comunicatie de date prin satelit", "",
	0x0F, 0x80, 0x00, "Alt controler de comunicatie prin satelit", "",

	0x10, 0x00, 0x00, "Controler de criptare si decriptare pentru retea si calculatoare", "",
	0x10, 0x10, 0x00, "Controler de criptare si decriptare pentru amuzament", "",
	0x10, 0x80, 0x00, "Alt controler de criptare si decriptare", "",

	0x11, 0x00, 0x00, "Modul Digital Parallel In Out (DPIO)", "",
	0x11, 0x01, 0x00, "Contoare de performanta", "",
	0x11, 0x10, 0x00, "Controler pentru sincronizarea comunicatiei, masurarea timpului si a frecventei", "",
	0x11, 0x20, 0x00, "Controler pentru gestiunea achizitei de date sau procesarii semnalelor", "",
	0x11, 0x80, 0x00, "Alt controler pentru achizitia de date sau procesarea semnalelor", "",

	0x12, 0x00, 0x00, "Accelerator de procesare ", "(interfata specifica producatorului)",
	0x12, 0x01, 0x00, "Accelerator pentru interfata IA", "",

	0x12, 0x01, 0x00, "Functie de instrumentatie ne-esentiala ", "(interfata specifica producatorului)",

	0xFF, 0x00, 0x00, "Dispozitiv cu cod neasignat al clasei/sub-clasei", "",
};

// Numarul de intrari in tabela; se poate utiliza pentru cautarea in tabela
#define	PCI_CLASS_TABLE_LEN	(sizeof(PciClassTable) / sizeof(PCI_CLASS_TABLE)) 

#endif
