#include <windows.h>

//-----------------------------------------------------------------------------------
// Functia PciBaseAddressUEFI
//-----------------------------------------------------------------------------------
// Functie:	    - Citeste continutul tabelei firmware ACPI MCFG
//				- Citeste din tabela ACPI MCFG adresa de baza a
//				  spatiului de configuratie PCI Express
//-----------------------------------------------------------------------------------
// Parametri:	-
// Returneaza:	Adresa de baza a spatiului de configuratie PCI Express
//				0 - Nu se poate determina adresa de baza
//-----------------------------------------------------------------------------------

DWORD64 PciBaseAddressUEFI(void)
{
    #pragma pack(push, 1)
    // Structura tabelei firmware ACPI MCFG
    typedef struct {
        BYTE    Signature[4];       // identificatorul tabelei (sir ASCII)
        DWORD   LengthTable;        // dimensiunea tabelei, in octeti
        BYTE    Revision;           // numarul reviziei tabelei
        BYTE    Checksum;           // suma de control pentru intreaga tabela
        BYTE    OEMID[6];           // sir care identifica producatorul OEM
        BYTE    OEMTableID[8];      // sir care identifica tabela OEM
        DWORD   OEMRevision;        // numarul reviziei OEM
        DWORD   CreatorID;          // identificatorul utilitarului care a creat tabela
        DWORD   CreatorRevision;    // revizia utilitarului care a creat tabela
        DWORD64 ReservedHeader;

        DWORD64 BaseAddress;        // adresa de baza ECAM (Enhanced Configuration Access Mechanism)
        WORD    PCISegmentGroup;    // numarul grupului segmentului PCI (implicit: 0)
        BYTE    StartBusNo;         // numarul primei magistrale decodificate de puntea calculatorului gazda
        BYTE    EndBusNo;           // numarul ultimei magistrale decodificate de puntea calculatorului gazda
        BYTE    Reserved[4];
    } ACPI_MCFG_TABLE;
    #pragma pack(pop)

    DWORD   BufferSize;
    DWORD64 BaseAddress;
    ACPI_MCFG_TABLE* pMcfgTable;

    // Determina dimensiunea necesara a bufferului pentru stocarea tabelei ACPI MCFG
    // Identificatorul tabelei firmware ('MCFG') trebuie sa fie in format "little endian",
    // astfel incat ordinea caracterelor din sirul de identificare este inversata ('GFCM')
    BufferSize = GetSystemFirmwareTable('ACPI', 'GFCM', NULL, 0);

    // Daca functia GetSystemFirmwareTable() esueaza, returneaza 0
    if (BufferSize == 0) {
        return 0;
    }

    // Aloca memorie pentru buffer
    pMcfgTable = (ACPI_MCFG_TABLE*)malloc(BufferSize);
    if (pMcfgTable == NULL) {
        return 0;               // eroare la alocarea memoriei
    }

    // Citeste tabela firmware ACPI MCFG
    BufferSize = GetSystemFirmwareTable('ACPI', 'GFCM', pMcfgTable, BufferSize);
    if (BufferSize == 0) {
        free(pMcfgTable);       // elibereaza memoria alocata
        return 0;               // functia GetSystemFirmwareTable() a esuat
    }

    // Citeste din buffer adresa de baza a spatiului de configuratie PCI Express
    BaseAddress = (DWORD64)(pMcfgTable->BaseAddress);
    free(pMcfgTable);           // elibereaza memoria alocata
    return BaseAddress;
}

